@@ -25,6 +25,7 @@ Warning: these tests should be fairly stable, but when writing/debugging new
 """
 
 import threading
+import os
 
 from dulwich.server import (
     DictBackend,
@@ -32,28 +33,14 @@ from dulwich.server import (
     )
 from dulwich.tests.compat.server_utils import (
     ServerTests,
-    ShutdownServerMixIn,
     NoSideBand64kReceivePackHandler,
     )
 from dulwich.tests.compat.utils import (
     CompatTestCase,
+    require_git_version,
     )
 
 
-if not getattr(TCPGitServer, 'shutdown', None):
-    _TCPGitServer = TCPGitServer
-
-    class TCPGitServer(ShutdownServerMixIn, TCPGitServer):
-        """Subclass of TCPGitServer that can be shut down."""
-
-        def __init__(self, *args, **kwargs):
-            # BaseServer is old-style so we have to call both __init__s
-            ShutdownServerMixIn.__init__(self)
-            _TCPGitServer.__init__(self, *args, **kwargs)
-
-        serve = ShutdownServerMixIn.serve_forever
-
-
 class GitServerTestCase(ServerTests, CompatTestCase):
     """Tests for client/server compatibility.
 
@@ -76,6 +63,7 @@ class GitServerTestCase(ServerTests, CompatTestCase):
                                   handlers=self._handlers())
         self._check_server(dul_server)
         self.addCleanup(dul_server.shutdown)
+        self.addCleanup(dul_server.server_close)
         threading.Thread(target=dul_server.serve).start()
         self._server = dul_server
         _, port = self._server.socket.getsockname()
@@ -88,6 +76,15 @@ class GitServerSideBand64kTestCase(GitServerTestCase):
     # side-band-64k in git-receive-pack was introduced in git 1.7.0.2
     min_git_version = (1, 7, 0, 2)
 
+    def setUp(self):
+        super(GitServerSideBand64kTestCase, self).setUp()
+        # side-band-64k is broken in the widows client.
+        # https://github.com/msysgit/git/issues/101
+        # Fix has landed for the 1.9.3 release.
+        if os.name == 'nt':
+            require_git_version((1, 9, 3))
+
+
     def _handlers(self):
         return None  # default handlers include side-band-64k
 
